@@ -44,12 +44,13 @@
 #include "gdict-enum-types.h"
 #include "gdict-marshal.h"
 #include "gdict-utils.h"
+#include "gdict-private.h"
 
 typedef enum {
   CMD_CLIENT,
   CMD_SHOW_DB,
   CMD_SHOW_STRAT,
-  CMD_SHOW_INFO, 	/* not implemented */
+  CMD_SHOW_INFO,        /* not implemented */
   CMD_SHOW_SERVER,	/* not implemented */
   CMD_MATCH,
   CMD_DEFINE,
@@ -191,19 +192,19 @@ static void gdict_client_context_get_property (GObject      *object,
 static void gdict_client_context_finalize     (GObject      *object);
 
 /* GdictContext methods */
-static gboolean gdict_client_context_get_databases  (GdictContext  *context,
-						     GError       **error);
-static gboolean gdict_client_context_get_strategies (GdictContext  *context,
-						     GError       **error);
-static gboolean gdict_client_context_define_word    (GdictContext  *context,
-						     const gchar   *database,
-						     const gchar   *word,
-						     GError       **error);
-static gboolean gdict_client_context_match_word     (GdictContext  *context,
-						     const gchar   *database,
-						     const gchar   *strategy,
-						     const gchar   *word,
-						     GError       **error);
+static gboolean gdict_client_context_get_databases     (GdictContext  *context,
+						        GError       **error);
+static gboolean gdict_client_context_get_strategies    (GdictContext  *context,
+						        GError       **error);
+static gboolean gdict_client_context_define_word       (GdictContext  *context,
+						        const gchar   *database,
+						        const gchar   *word,
+						        GError       **error);
+static gboolean gdict_client_context_match_word        (GdictContext  *context,
+						        const gchar   *database,
+						        const gchar   *strategy,
+						        const gchar   *word,
+						        GError       **error);
 
 static void     gdict_client_context_clear_hostinfo    (GdictClientContext  *context);
 static gboolean gdict_client_context_lookup_server     (GdictClientContext  *context,
@@ -226,12 +227,7 @@ static void          gdict_command_free (GdictCommand     *cmd);
 GQuark
 gdict_client_context_error_quark (void)
 {
-  static GQuark quark = 0;
-  
-  if (G_UNLIKELY (quark == 0))
-    quark = g_quark_from_static_string ("gdict-client-context-error-quark");
-  
-  return quark;
+  return g_quark_from_static_string ("gdict-client-context-error-quark");
 }
 
 static void
@@ -711,7 +707,7 @@ gdict_client_context_push_command (GdictClientContext *context,
       return FALSE;
     }
   
-  gdict_debug ("Pushing command ('%s') into the queue...\n",
+  _gdict_debug ("Pushing command ('%s') into the queue...\n",
                dict_command_strings[command->cmd_type]);
   g_queue_push_head (priv->commands_queue, command);
   
@@ -732,7 +728,7 @@ gdict_client_context_pop_command (GdictClientContext *context)
   if (!retval)
     return NULL;
   
-  gdict_debug ("Getting command ('%s') from the queue...\n",
+  _gdict_debug ("Getting command ('%s') from the queue...\n",
                dict_command_strings[retval->cmd_type]);
   
   return retval;
@@ -756,7 +752,7 @@ gdict_client_context_send_command (GdictClientContext  *context,
   
   if (!priv->channel)
     {
-      gdict_debug ("No connection established\n");
+      _gdict_debug ("No connection established\n");
       
       g_set_error (error, GDICT_CLIENT_CONTEXT_ERROR,
                    GDICT_CLIENT_CONTEXT_ERROR_NO_CONNECTION,
@@ -783,7 +779,7 @@ gdict_client_context_send_command (GdictClientContext  *context,
   /* force flushing of the write buffer */
   g_io_channel_flush (priv->channel, NULL);
   
-  gdict_debug ("Wrote (%d bytes) to the channel\n",
+  _gdict_debug ("Wrote (%d bytes) to the channel\n",
                written_bytes);
   
   return TRUE;
@@ -806,7 +802,7 @@ gdict_client_context_run_command (GdictClientContext  *context,
   g_assert (command != NULL);
   g_assert (IS_VALID_CMD (command->cmd_type));
   
-  gdict_debug ("GdictCommand command =\n"
+  _gdict_debug ("GdictCommand command =\n"
                "{\n"
                "  .cmd_type = '%02d' ('%s');\n"
                "  .database = '%s';\n"
@@ -814,9 +810,9 @@ gdict_client_context_run_command (GdictClientContext  *context,
                "  .word     = '%s';\n"
                "}\n",
                command->cmd_type, dict_command_strings[command->cmd_type],
-               command->database,
-               command->strategy,
-               command->word);
+               command->database ? command->database : "<none>",
+               command->strategy ? command->strategy : "<none>",
+               command->word ? command->word : "<none>");
   
   priv = context->priv;
 
@@ -866,7 +862,7 @@ gdict_client_context_run_command (GdictClientContext  *context,
   
   g_assert (command->cmd_string);
 
-  gdict_debug ("Sending command ('%s') to the server\n",
+  _gdict_debug ("Sending command ('%s') to the server\n",
                dict_command_strings[command->cmd_type]);
 
   send_error = NULL;
@@ -1000,14 +996,14 @@ gdict_client_context_lookup_server (GdictClientContext  *context,
   /* clear any previously acquired lookup data */
   gdict_client_context_clear_hostinfo (context);
   
-  gdict_debug ("Looking up hostname '%s'\n", priv->hostname);
+  _gdict_debug ("Looking up hostname '%s'\n", priv->hostname);
   
 #ifdef ENABLE_IPV6
-  if (gdict_has_ipv6 ())
+  if (_gdict_has_ipv6 ())
     {
       struct addrinfo hints, *res;
       
-      gdict_debug ("Hostname '%s' look-up (using IPv6)\n", priv->hostname);
+      _gdict_debug ("Hostname '%s' look-up (using IPv6)\n", priv->hostname);
       
       memset (&hints, 0, sizeof (hints));
       hints.ai_socktype = SOCK_STREAM;
@@ -1041,7 +1037,8 @@ gdict_client_context_lookup_server (GdictClientContext  *context,
 
 	      priv->sockaddr.ss_family = res->ai_family;
 	      
-	      gdict_debug ("Hostname '%s' found (using IPv6)\n", priv->hostname);
+	      _gdict_debug ("Hostname '%s' found (using IPv6)\n",
+			    priv->hostname);
 	      
 	      priv->last_lookup = time (NULL);
 
@@ -1064,7 +1061,8 @@ gdict_client_context_lookup_server (GdictClientContext  *context,
 #endif /* ENABLE_IPV6 */
       /* if we don't support IPv6, fallback to usual IPv4 lookup */
       
-      gdict_debug ("Hostname '%s' look-up (using IPv4)\n", priv->hostname);
+      _gdict_debug ("Hostname '%s' look-up (using IPv4)\n",
+		    priv->hostname);
       
       ((struct sockaddr_in *) &priv->sockaddr)->sin_family = AF_INET;
 	
@@ -1075,7 +1073,8 @@ gdict_client_context_lookup_server (GdictClientContext  *context,
                   priv->hostinfo->h_addr,
                   priv->hostinfo->h_length);
           
-          gdict_debug ("Hostname '%s' found (using IPv4)\n", priv->hostname);
+          _gdict_debug ("Hostname '%s' found (using IPv4)\n",
+			priv->hostname);
 
 	  priv->last_lookup = time (NULL);
           
@@ -1115,7 +1114,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
   
   priv = context->priv;
   
-  gdict_debug ("parse buffer: '%s'\n", buffer);
+  _gdict_debug ("parse buffer: '%s'\n", buffer);
   
   /* connection is a special case: we don't have a command, so we just
    * make sure that the server replied with the correct code. WARNING:
@@ -1188,7 +1187,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("No definitions found for '%s'"),
                    priv->command->word);
 
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
           
@@ -1203,7 +1202,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("Invalid database '%s'"),
                    priv->command->database);
 
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
           
@@ -1218,7 +1217,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("Invalid strategy '%s'"),
                    priv->command->strategy);
       
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
           
       g_signal_emit_by_name (context, "error", server_error);
           
@@ -1233,7 +1232,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("Bad command '%s'"),
                    dict_command_strings[priv->command->cmd_type]);
       
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
       
@@ -1248,7 +1247,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
 		   _("Bad parameters for command '%s'"),
 		   dict_command_strings[priv->command->cmd_type]);
 
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
 
@@ -1263,7 +1262,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("No databases found on dictionary server at '%s'"),
                    priv->hostname);
           
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
       
@@ -1278,7 +1277,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
                    _("No strategies found on dictionary server at '%s'"),
                    priv->hostname);
           
-      gdict_debug ("error: %s\n", server_error->message);
+      _gdict_debug ("error: %s\n", server_error->message);
       
       g_signal_emit_by_name (context, "error", server_error);
           
@@ -1288,7 +1287,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
       priv->command->state = S_FINISH;
       break;
     default:
-      gdict_debug ("non error code: %d\n", priv->status_code);
+      _gdict_debug ("non error code: %d\n", priv->status_code);
       break;
     }
 
@@ -1335,7 +1334,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
       return TRUE;
     }
 
-  gdict_debug ("check command %d ('%s')[state:%d]\n",
+  _gdict_debug ("check command %d ('%s')[state:%d]\n",
 	       priv->command->cmd_type,
 	       dict_command_strings[priv->command->cmd_type],
 	       priv->command->state);
@@ -1357,9 +1356,9 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (p)
             p = g_utf8_next_char (p);
           
-          gdict_debug ("server replied: %d databases found\n", atoi (p));
+          _gdict_debug ("server replied: %d databases found\n", atoi (p));
           
-	  gdict_debug ("emitting lookup-start\n");
+	  _gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1394,7 +1393,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           db = _gdict_database_new (name);
           db->full_name = g_strdup (full);
           
-	  gdict_debug ("emitting database-found\n");
+	  _gdict_debug ("emitting database-found\n");
           g_signal_emit_by_name (context, "database-found", db);
           
           gdict_database_unref (db);
@@ -1411,9 +1410,9 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (p)
             p = g_utf8_next_char (p);
           
-          gdict_debug ("server replied: %d strategies found\n", atoi (p));
+          _gdict_debug ("server replied: %d strategies found\n", atoi (p));
           
-	  gdict_debug ("emitting lookup-start\n");
+	  _gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1445,7 +1444,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           strat = _gdict_strategy_new (name);
           strat->description = g_strdup (desc);
           
-	  gdict_debug ("emitting strategy-found\n");
+	  _gdict_debug ("emitting strategy-found\n");
           g_signal_emit_by_name (context, "strategy-found", strat);
           
           gdict_strategy_unref (strat);
@@ -1463,14 +1462,14 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (p)
             p = g_utf8_next_char (p);
           
-          gdict_debug ("server replied: %d definitions found\n", atoi (p));
+          _gdict_debug ("server replied: %d definitions found\n", atoi (p));
           
           def = _gdict_definition_new (atoi (p));
           
           priv->command->data = def;
           priv->command->data_destroy = (GDestroyNotify) gdict_definition_unref;
 
-	  gdict_debug ("emitting lookup-start\n");
+	  _gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (priv->status_code == GDICT_STATUS_WORD_DB_NAME)
@@ -1517,7 +1516,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           
           def = (GdictDefinition *) priv->command->data;
 
-	  gdict_debug ("{ word .= '%s', db_name .= '%s', db_full .= '%s' }\n",
+	  _gdict_debug ("{ word .= '%s', db_name .= '%s', db_full .= '%s' }\n",
 		       word,
 		       db_name,
 		       db_full);
@@ -1545,7 +1544,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
 	  /* store the numer of definitions */
 	  num = def->total;
           
-	  gdict_debug ("emitting definition-found\n");
+	  _gdict_debug ("emitting definition-found\n");
           g_signal_emit_by_name (context, "definition-found", def);
           
           gdict_definition_unref (def);
@@ -1562,7 +1561,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (!priv->command->buffer)
             priv->command->buffer = g_string_new (NULL);
           
-	  gdict_debug ("appending line:\n %s\n", buffer);
+	  _gdict_debug ("appending line:\n %s\n", buffer);
           
           /* TODO - collapse '..' to '.' */
           g_string_append_printf (priv->command->buffer, "%s\n", buffer);
@@ -1579,9 +1578,9 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (p)
             p = g_utf8_next_char (p);
           
-          gdict_debug ("server replied: %d matches found\n", atoi (p));
+          _gdict_debug ("server replied: %d matches found\n", atoi (p));
 
-	  gdict_debug ("emitting lookup-start\n");
+	  _gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1613,7 +1612,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           match = _gdict_match_new (word);
           match->database = g_strdup (db_name);
           
-	  gdict_debug ("emitting match-found\n");
+	  _gdict_debug ("emitting match-found\n");
           g_signal_emit_by_name (context, "match-found", match);
           
           gdict_match_unref (match);
@@ -1629,10 +1628,11 @@ gdict_client_context_parse_line (GdictClientContext *context,
 
 /* retrieve the status code from the server response line */
 static gint
-get_status_code (const gchar *line)
+get_status_code (const gchar *line,
+		 gint         old_status)
 {
   gchar *status;
-  gint retval;
+  gint possible_status, retval;
   
   if (strlen (line) < 3)
     return 0;
@@ -1641,14 +1641,42 @@ get_status_code (const gchar *line)
       !g_unichar_isdigit (line[1]) ||
       !g_unichar_isdigit (line[2]))
     return 0;
+
+  if (!g_unichar_isspace (line[3]))
+    return 0;
   
   status = g_strndup (line, 3);
-  retval = atoi (status);
+  possible_status = atoi (status);
   g_free (status);
+
+  /* status whitelisting: sometimes, a database *cough* moby-thes *cough*
+   * might return a number as first word; we do a small check here for
+   * invalid status codes based on the previously set status; we don't check
+   * the whole line, as we need only to be sure that the status code is
+   * consistent with what we expect.
+   */
+  switch (old_status)
+    {
+    case GDICT_STATUS_WORD_DB_NAME:
+    case GDICT_STATUS_N_MATCHES_FOUND:
+      if (possible_status == GDICT_STATUS_OK)
+	retval = possible_status;
+      else
+        retval = 0;
+      break;
+    case GDICT_STATUS_N_DEFINITIONS_RETRIEVED:
+      if (possible_status == GDICT_STATUS_WORD_DB_NAME)
+	retval = possible_status;
+      else
+	retval = 0;
+      break;
+    default:
+      retval = possible_status;
+      break;
+    }
   
   return retval;
 }  
-  
 
 static gboolean
 gdict_client_context_io_watch_cb (GIOChannel         *channel,
@@ -1735,12 +1763,12 @@ gdict_client_context_io_watch_cb (GIOChannel         *channel,
       /* truncate the line terminator before parsing */
       line[term] = '\0';
       
-      status_code = get_status_code (line);
+      status_code = get_status_code (line, priv->status_code);
       if ((status_code == 0) || (GDICT_IS_VALID_STATUS_CODE (status_code)))
         {
           priv->status_code = status_code;
           
-          gdict_debug ("new status = '%d'\n", priv->status_code);
+          _gdict_debug ("new status = '%d'\n", priv->status_code);
         }
       else
         priv->status_code = GDICT_STATUS_INVALID;
@@ -1754,14 +1782,14 @@ gdict_client_context_io_watch_cb (GIOChannel         *channel,
         {
           g_free (line);
           
-          gdict_debug ("Parse failed\n");
+          _gdict_debug ("Parse failed\n");
           
           gdict_client_context_force_disconnect (context);
           
           return FALSE;
         }
       
-      gdict_debug ("Line parsed\n");
+      _gdict_debug ("Line parsed\n");
       
       g_free (line);
     }
@@ -1925,7 +1953,7 @@ gdict_client_context_is_connected (GdictClientContext *context)
   g_assert (GDICT_IS_CLIENT_CONTEXT (context));
   
   /* we are in the middle of a connection attempt */
-  if (context->priv->is_connecting == TRUE)
+  if (context->priv->is_connecting)
     return TRUE;
   
   return (context->priv->channel != NULL && context->priv->source_id != 0);
